#!/usr/bin/perl
#
# make_nulldistri.pimut_cond_ctfwmuts_cum.pl:
#	This Perl script was made by slightly modifying the script
#	'make_nulldistri.pimut_cond_ctmuthaplos_cum.pl'.
#	It constructs null-distributions of pi(muts} conditional on
#	the average #{forward_mutations}.
#
#	NOTE: Throughout this script, the labeling 'ct_muthaplos' for the variables
#	actually means 'ct_fw_muts'.
#
#	(CREATED on Feb 24 (Fri), 2012)
#
# make_nulldistri.pimut_cond_ctmuthaplos_cum.pl:
#	This is identical to 'make_nulldistri.pimut_cum.pl',
#	but was renamed so that the name reflects more precisely what it outputs.
#	(RENAMED on Feb 5 (Sun), 2012)
#
# make_nulldistri.pimut_cum.pl:
#	This script was created by substantially modifying 'summarize_br_pars_msms.3pimut_cum.pl'.
#	This script constructs NULL DISTRIBUTIONS, i.e. relative cumulative frequencies
#	under several null hypotheses, of pi(muts), as well as giving their weights.
#	NULL HYPOTHESES are defined by two sets of weight factors:
#	(i) Weight factors for the forward mutation rate, Smu, which follows the
#	power-law distribution: Pr [ Smu >= x ] = B * {x^{-$ALPHA} - (Smu_MAX)^{-$ALPHA}},
#	with a fixed power $ALPHA.
#	(ii) Weight factors for the ratio of forward and backward mutation rates, R = Snu/Smu.
#	We consider the following 4 combinations:
#	(1,2,3) wgt(R) = 1 for a fixed R (= 0, 1, or 3), = 0 otherwise;
#	(4) wgt(R) = 1 for all R = 0, 1/2, 1, 2, 3.
#	(CREATION STARTED on Jan 28 (Sat), 2012, ENDED on Jan (), 2012)
#
# summarize_br_pars_msms.3pimut_cum.pl:
#	This script was created by modifying 'summarize_br_pars_msms.3pimut.pl'.
#	It juxtaposes relative cumulative frequencies of pi(muts),
#	instead of their weights as the predecessor does.
#	(CREATED on Jan 25 (Wed), 2012)
#
# summarize_br_pars_msms.3pimut.pl:
#	This Perl script was created by modifying the script 'summarize_br_pars_msms.3mut_frqcmnst.pl'.
#	It handles the average pairwise sequence difference among MUTANT samples,
#	which will be used for Teshima&Innan's test to detect negative selection on recurrent mutations.
#	(CREATED on Jan 25 (Wed), 2012)
#
# summarize_br_pars_msms.3mut_frqcmnst.pl:
#	This Perl script was created by modifying the script 'summarize_br_pars_msms.3ewens.pl'.
#	It handles the frequency of the commonest MUTANT allele conditional on the number of MUTANT alleles,
#	that is used for a new test to detect negative selection on recurrent mutations.
#	(CREATED on Jan 23 (Mon), 2012)
#
# summarize_br_pars_msms.3ewens.pl:
#	This revised version of 'summarize_br_pars_msms.3max.pl'
#	handles the frequency of the commonest allele conditional on the number of alleles,
#	that is  necessary for Ewens' test.
#	(CREATED on Jan 22 (Sun), 2012)
#
# summarize_br_pars_msms.3max.pl:
#	This is a slightly modified version of 'summarize_br_pars_msms.3tot.pl'
#	that handles the maximum#{mutants}, instead of total#{mutants}.
#	(CREATED on Jan 4 (Wed), 2012)
#
# summarize_br_pars_msms.3tot.pl:
#	This is created by slightly modifying 'summarize_br_pars_msms.3tot_cum.pl'
#	so that the original absolute frequencies of total#{mutants} should be output,
#	instead of their relative cumulative frequencies.
#	(CREATED on Jan 3 (Tue), 2012)
#
# summarize_br_pars_msms.3tot_cum.pl:
#	This script juxtaposes weights for different Snu's
#	and for shared total#{mutants}, under a fixed value of sAA,
#	separately for different #{forward mutations}, and for different values of Smu.
#	(CREATED on Jan 3 (Tue), 2012)
#
# summarize_br_pars_msms.2tot_cum.pl:
#	This script is a slightly modified version of 'summarize_br_pars_msms.2max.pl'.
#	It handles the total#{mutants}, instead of maximum#{mutants} as the predecessor does.
#	(CREATED on Jan 1 (Sun), 2012)
#
# summarize_br_pars_msms.2max_cum.pl:
#	This script was created by slightly modifying 'summarize_br_pars_msms.2max.pl'.
#	It outputs cumulative relative frequencies, instead of raw absolute frequencies.
#	(CREATED on Jan 1 (Sun), 2012)
#
# summarize_br_pars_msms.2max.pl:
#	This Perl script was created by slightly modifying 'summarize_br_pars_msms.2tot.pl',
#	and it handles maximum#{mutants}, instead of total#{mutants} as the predecessor does.
#	[NOTE: Although the variables bear the name 'totct_muts',
#	they actually refer to the maximum number of mutants descendedn from a mutation event. ]
#	(CREATED on Dec 31 (Sat), 2011)
#
# summarize_br_pars_msms.2tot.pl:
#	This Perl script juxtaposes weights for different sAA
#	and for shared total#{mutants},
#	separately for different #{forward mutations} (within the same file),
#	for different #{samples} (in different files), and for different Smu & Snu (in different directries).
#	(CREATED on Dec 31 (Sat), 2011)
#
# summarize_br_pars_msms.1.pl:
#	This Perl script summarizes the outputs of 'ser_br_pars_out_msms.1.pl'
#	and of 'hyperser_br_pars_out_msms.1.pl'.
#	(CREATION BEGAN on Dec 29 (Thu), 2011, ENDED on Dec 30 (Fri), 2011)
#

use strict;
use warnings;

use MyFile_manipu qw( open_input_file2 open_output_file list_sessions_ext tfm_labels2indices );
use MyArrayManipu qw( multiple_sort );
##use MyArrayManipu_num qw( binsearch_num );

sub make_weights_smu (\@$$$);


$| = 1; # Flush the buffer for the standard output everytime data flows into it. #

my $plscript_path = $0;
my ($plscript_name)  = ($plscript_path =~ m!([^/]+)$!);
unless ((defined $plscript_name) and ($plscript_name =~ m/\S/)) {
     print STDERR "[ERROR] FAILED to EXTRACT a SCRIPT NAME from the path '$plscript_path' !\n";
	print STDERR "[ERROR '$plscript_path'] $!\n";
	die ;
}

	### Parameters to be adjusted before running the script. ###

our $LB_SMD_WGT = 40; # Lower bound of the summed weight for the category to be accepted into the null distribution.
our ($SMU_MIN, $SMU_MAX) = (0.0316, 100);
##our $POWER_SMU = 0.5; # This specifies the power-law for the weight factors: Prb [ Smu >= x] = A * x**{-$POWER_SMU}.
##our $POWER_SMU = 1; # This specifies the power-law for the weight factors: Prb [ Smu >= x] = A * x**{-$POWER_SMU}.
our $POWER_SMU = 2; # This specifies the power-law for the weight factors: Prb [ Smu >= x] = A * x**{-$POWER_SMU}.

=for the predecessor 'summarize_br_pars_msms.1.pl'
our @FLD_LABELS1 = ('#Rep_ID', '#{chromosomes}', '#{neut_seg_sites}', 'fin_freq', '#{parsim_scenarios}');
our @FLD_LABELS2 = ('Pars_scen_ID',  'ancestral_state', "#{WT(eq'0')}", "#{mutants(ne'0')}", '#{parsim_chgs}', '#{muts(0->others)}', '#{muts(others->0)}', '#IBD,descendants,of,fw,mutants', '#IBD,descendants,of,bw,mutants');
=cut


	# Basically, sAA should be ZERO, because we are calculating the null hypothesis.
our $LABEL_SAA = '0';
=for the predecessor 'summarize_br_pars_msms.3pimut_cum.pl'
##our $LABEL_SAA = 'neg10';
##our $LABEL_SAA = 'neg31.6';
##our $LABEL_SAA = 'neg100';
##our $LABEL_SAA = 'neg316';
=cut

##=for full-scale analyses 
	# REVISED in creation of 'make_nulldistri.pimut_cond_ctfwmuts_cum.pl' on Feb 24 (Fri), 2012. #
our @LABELS_SNU = (0, 'Smu', '3Smu');
our @SMUs = (0.1, 0.316, 1, 3.16, 10);
our @SSIZES = (20, 50, 100, 200);

	## Weight factors ##
	## (ADDED in creation of 'make_nulldistri.pimut_cum.pl' (on Jan 28 (Sat), 2012)) #

our @SETS_WEIGHTS_RATIOS = (
	[1, 1, 1],
	[1, 0, 0],
	[0, 1, 0],
	[0, 0, 1],
	);
our @NAMES_SETS_WEIGHTS_RATIOS = ('Even', @LABELS_SNU);
##=cut
=for full-scale analyses # for the predecessor 'make_nulldistri.pimut_cond_ctmuthaplos_cum.pl'
##our @LABELS_SNU = (0, 'Smu', '3Smu'); # For sAA < 0.
our @LABELS_SNU = (0, 'hfSmu', 'Smu', '2Smu', '3Smu', '10Smu'); # For sAA  == 0
our @SMUs = (0.1, 0.316, 1, 3.16, 10);
our @SSIZES = (20, 50, 100, 200);

	## Weight factors ##
	## (ADDED in creation of 'make_nulldistri.pimut_cum.pl' (on Jan 28 (Sat), 2012)) #

our @SETS_WEIGHTS_RATIOS = (
	[1, 1, 1, 1, 1, 0],
	[1, 0, 0, 0, 0, 0],
	[0, 0, 1, 0, 0, 0],
	[0, 0, 0, 0, 1, 0],
	[0, 0, 0, 0, 0, 1]
	);
our @NAMES_SETS_WEIGHTS_RATIOS = ('Even', @LABELS_SNU[0, 2, 4, 5]);
=cut

=for TESTING
##@LABELS_SNU = ( 0 ); # Testing
##@LABELS_SNU = ( 'hfSmu' ); # Testing
## @LABELS_SNU = ( 'Smu' ); # Testing 1
## @LABELS_SNU = ( '2Smu' ); # Testing 1
## @LABELS_SNU = (0, 'Smu', '3Smu'); # Testing 2
##@SMUs = ( 0.1 );
@SMUs = (1, 3.16);
## @SMUs = (10);
@SSIZES = (20, 200);
##@SETS_WEIGHTS_RATIOS = (
##	[1],
##	[2]
##	);
##@NAMES_SETS_WEIGHTS_RATIOS = ($LABELS_SNU[0], "Double_$LABELS_SNU[0]");
$LB_SMD_WGT = 2; # Lower bound of the summed weight for the category to be accepted into the null distribution.
=cut

our $CT_SNUS = @LABELS_SNU;
our $CT_SMUs = @SMUs;
our $CT_SETS_WGT_RATIOS = @SETS_WEIGHTS_RATIOS;

	### Input/OUtput directories/files ###

	# $infile = "$indir/$infile_pref$size$infile_suff", with
	# $indir = "$intopdir/$indir_pref1$label_snu/$indir_pref2$label_smu/$indir_pref3$label_sAA".
	#
=for the predecessor 'summarize_br_pars_msms.3max.pl'
##my $intopdir = "../Summary1/TESTs/Test1c1a"; # For TESTING
##my $intopdir = "../Summary1/TESTs/Test1a1"; # For TESTING
my $intopdir = "../Summary1/FullScale"; # For full-scale analyses. 
=cut
=for the predecessor 'summarize_br_pars_msms.3ewens.pl'
##my $intopdir = "../Summary1wh/TESTs/Test1c3"; # For TESTING
my $intopdir = "../Summary1wh/FullScale"; # For full-scale analyses.
=cut
=for the predecessor 'make_nulldistri.pimut_cond_ctmuthaplos_cum.pl'
##my $intopdir = "../../Summary1mut/TESTs/Test1c2"; # For TESTING
my $intopdir = "../../Summary1mut/FullScale"; # For full-scale analyses.
=cut
##my $intopdir = "../../Summary1mut_comb/TESTs/TestB"; # For TESTING
my $intopdir = "../../Summary1mut_comb/FullScale"; # For full-scale analyses.

my $indir_pref1 = 'Snu_';
my $indir_pref2 = 'Smu_';
my $indir_pref3 = 'SAA_';
=for the predecessor 'summarize_br_pars_msms.3max.pl'
##my $infile_pref = 'stat_totmutcts_size';
my $infile_pref = 'stat_maxmutcts_size';
=cut
=for the predecessor 'summarize_br_pars_msms.3ewens.pl'
my $infile_pref = 'stat_ewens_test_size';
=cut
	# MAJOR REVISION in creation of 'summarize_br_pars_msms.3pimut.pl' (on Jan 25 (Wed), 2012) #
my $infile_pref = 'stat_pimut_size';
##my $infile_pref = 'stat_frqcommonestmut_size';
my $infile_suff = '.txt.gz';

	# $outfile = "$outdir/$outfile_pref$size$outfile_suff", and
	# [ $outdir = "$outtopdir/$outdir_pref1$LABEL_SAA/$outdir_pref2$label_smu". ] --- for the predecessor 'summarize_br_pars_msms.3pimut_cum.pl'.
	# $outdir = "$outtopdir/$outdir_pref1$LABEL_SAA".
	#
##my $outtopdir = sprintf ("TESTs/TestB1/Alpha_%3.1f", $POWER_SMU); # For TESTING
##my $outtopdir = sprintf ("TESTs/TestC/Alpha_%3.1f", $POWER_SMU); # For TESTING
my $outtopdir = sprintf ("Alpha_%3.1f", $POWER_SMU); # For full-scale analyses.
## [ my $outtopdir = "FullScale"; # For full-scale analyses. ]  --- for the predecessor 'summarize_br_pars_msms.3pimut_cum.pl'.
my $outdir_pref1 = 'SAA_';
=for the predecessor 'summarize_br_pars_msms.3pimut_cum.pl'
my $outdir_pref2 = 'Smu_';
##my $outdir_pref3 = $indir_pref3;
=cut
=for the predecessor 'summarize_br_pars_msms.3max.pl'
##my $outfile_pref = 'tbls_totmutcts_jxt_Snu_size';
my $outfile_pref = 'tbls_maxmutcts_jxt_Snu_size';
=cut
my $outfile_pref = 'nulldistri_pimut_size';
##my $outfile_pref = 'nulldistri_frqcommonestmut_size';
##my $outfile_pref = 'nulldistri_ewens_size';
my $outfile_suff = '.txt.gz';

my $outfile2_pref = 'ingredients_pimut_size'; ## ADDED in creation of 'make_nulldistri.pimut_cum.pl' on Jan 29 (Sun), 2012. ##

	### Check input/output directories/files ###

unless ((-d $intopdir) and ( -r $intopdir) ) {
	print STDOUT "[ERROR '$plscript_name'] CanNOT FIND a readable TOP-TOP INPUT DIRECTORY '$intopdir' !\n";
	print STDERR "[ERROR '$plscript_name'] $! !\n\n";
	die ;
}

foreach my $label_snu (@LABELS_SNU) {
    my $inmiddir1 = "$intopdir/$indir_pref1$label_snu";
    unless (( -d $inmiddir1 ) and ( -r $inmiddir1 )) {
	print STDOUT "[ERROR '$plscript_name'] CanNOT FIND a readable 1st-rank MIDDLE INPUT DIRECTORY '$inmiddir1' !\n";
	print STDERR "[ERROR '$plscript_name'] $! !\n\n";
	die ;
    }
    foreach my $label_smu (@SMUs) {
	my $inmiddir2 = "$inmiddir1/$indir_pref2$label_smu";

	foreach my $label_sAA ($LABEL_SAA) {
	    my $indir = "$inmiddir2/$indir_pref3$label_sAA";
	    unless (( -d $indir ) and ( -r $indir )) {
		print STDOUT "[ERROR '$plscript_name'] CanNOT FIND a readable INPUT DIRECTORY '$indir' !\n";
		print STDERR "[ERROR '$plscript_name'] $! !\n\n";
		die ;
	    }
	    foreach  my $ssize (@SSIZES) {
		my $infile_name = "$infile_pref$ssize$infile_suff";
		my $infile = "$indir/$infile_name";
		unless (( -f $infile ) and ( -r $infile )) {
		    print STDOUT "[ERROR '$plscript_name'] CanNOT FIND a readable INPUT FILE named '$infile_name' in the directory '$indir' !\n";
		    print STDERR "[ERROR '$plscript_name'] $! !\n\n";
		    die ;
		}
	    }		# End of foreach over sample sizes #
	}		# End of foreach over sAA's #
    }			# End of foreach over Smu's #
}			# End of foreach over Snu's #


unless (-d $outtopdir) {
    unless (0 == system ("mkdir -p $outtopdir") ) {
	print STDOUT "[ERROR '$plscript_name'] FAILED to CREATE the TOP-TOP OUTPUT DIRECTORY '$outtopdir': $? !\n";
	print STDERR "[ERROR '$plscript_name'] $! !\n\n";
	die ;
    }
}
unless ((-r $outtopdir) and (-w $outtopdir)) {
    unless (0 == system ("chmod -R u+rw $outtopdir") ) {
	print STDOUT "[ERROR '$plscript_name'] FAILED to CHANGE the permission MODES of the directory '$outtopdir': $? !\n";
	print STDERR "[ERROR '$plscript_name'] $! !\n";
	die ;
    }
}

	# Calculate weight factors for the Smu's #
	# (ADDED in creation of 'make_nulldistri.pimut_cum.pl' on Jan 28 (Sat), 2012) #

my $weights_smu = make_weights_smu (@SMUs, $SMU_MIN, $SMU_MAX, $POWER_SMU);
unless (defined $weights_smu) {
	print STDOUT "[ERROR '$plscript_name'] 'make_weights_smu' FAILED !\n";
	print STDERR "[ERROR '$plscript_name'] $! !\n";
	die ;
}


	## Declare the Beginning of the Program ##

print STDOUT "\n<<<<< Program name: $plscript_name (path: $plscript_path) >>>>>\n\n";

print STDOUT "Input files: {in_dir}/$infile_pref\{ssize\}$infile_suff,\n";
print STDOUT "        where {in_dir} = {intop_dir}/$indir_pref1\{label_snu\}/$indir_pref2\{label_smu\}/$indir_pref3\{label_sAA\} ,\n";
print STDOUT "        with {intop_dir} = $intopdir ,\n";
print STDOUT "             {label_snu} = ", join (', ', @LABELS_SNU), " ,\n";
print STDOUT "             {label_smu} = ", join (', ', @SMUs), " ,\n";

print STDOUT "             {label_sAA} = $LABEL_SAA ,\n";
print STDOUT "              {ssize}    = ", join (', ', @SSIZES), " .\n";
print STDOUT "\n\n";

	# REVISED in creation of 'make_nulldistri.pimut_cum.pl' (on Jan 29 (Sun), 2012) #
print STDOUT "Output files:\n";
print STDOUT "  (Main)       {out_dir}/$outfile_pref\{ssize\}$outfile_suff,\n";
print STDOUT "  (Auxiliary)  {out_dir}/$outfile2_pref\{ssize\}$outfile_suff,\n";
##print STDOUT "Output files: {out_dir}/$outfile_pref\{ssize\}$outfile_suff,\n";

	# REVISED in creation of 'make_nulldistri.pimut_cum.pl' (on Jan 28 (Sat), 2012) #
print STDOUT "        where {out_dir} = {outtop_dir}/$outdir_pref1\{label_sAA\} ,\n";
print STDOUT "        with {outtop_dir} = $outtopdir ,\n";
print STDOUT "        ({label_sAA}, and {ssize} will be the same as those for the input files).\n\n";
=for the predecessor 'summarize_br_pars_msms.3pimut_cum.pl'
print STDOUT "        where {out_dir} = {outtop_dir}/$outdir_pref1\{label_sAA\}/$outdir_pref2\{label_smu\} ,\n";
print STDOUT "        with {outtop_dir} = $outtopdir ,\n";
print STDOUT "        ({label_smu}, {label_sAA}, and {ssize} will be the same as those for the input files).\n\n";
=cut

	# ADDED  in creation of 'make_nulldistri.pimut_cum.pl' on Jan 29 (Sun), 2012 #
print STDOUT "<<<< Condition on each class to be used for a null-distribution >>>>\n\n";
print STDOUT "{Total summation of weights over the class} >= \$LB_SMD_WGT = $LB_SMD_WGT .\n\n";

	# Print out weight ractors #
	# (ADDED  in creation of 'make_nulldistri.pimut_cum.pl' on Jan 28 (Sat), 2012) #

print STDOUT "\n<<<< Rules for Weight factors of Smu's >>>>\n\n";

print STDOUT "Weight factor is based on the power-law: Pr (Smu >= x) = A * x**(-$POWER_SMU) ,\n";
print STDOUT " with the boundaries (Smu_min, Smu_Max) = ($SMU_MIN, $SMU_MAX)\n\n";


print STDOUT "\n<<<< Weight factors for the Smu's >>>>\n\n";

print STDOUT join ("\t", 'Smu', 'Weight'), "\n\n";
for (my $m=0; $m<$CT_SMUs; $m++) { print STDOUT join ("\t", $SMUs[$m], $weights_smu->[$m]), "\n"; }
print STDOUT "\n\n";

print STDOUT "\n<<<< Sets of Weight factors for the Snu/Smu ratios >>>>\n\n";

print STDOUT join ("'\t'", "'Set_name'\\'label_Snu", @LABELS_SNU), "\n\n";

for (my $c=0; $c<$CT_SETS_WGT_RATIOS; $c++) { # Preliminary for-loop #
	print STDOUT join ("\t", "'$NAMES_SETS_WEIGHTS_RATIOS[$c]'", @{$SETS_WEIGHTS_RATIOS[$c]}), "\n";
}	# End of the preliminary for-loop #
print STDOUT "\n\n";


print STDOUT "\n<<<<< Now, the Main Processes will BEGIN !!! >>>>>\n\n";

foreach my $label_sAA ($LABEL_SAA) { # Main outermost foreach-loop (over a single sAA) #

    print STDOUT "\n<<<< sAA:  $label_sAA >>>>\n\n";

		# MODIFIED in creation of 'make_nulldistri.pimut_cum.pl' (on Jan 28 (Sat), 2012) #
    my $outdir = "$outtopdir/$outdir_pref1$label_sAA";
    ##my $outdir = "$outtopdir/$outdir_pref1$label_sAA/$outdir_pref2$label_smu";
    print STDOUT "Output directory: $outdir .\n\n";

    unless ( -d $outdir ) {
	unless (0 == system ("mkdir -p $outdir") ) {
		print STDOUT "[ERROR '$plscript_name'] FAILED to CREATE an output directory '$outdir' !\n";
		print STDERR "[ERROR '$plscript_name'] $! \n";
		die ;
	}
     }
     unless (( -r $outdir ) and ( -w $outdir )) {
	unless (0 == system ("chmod -R u+rw $outdir" ) ) {
		print STDOUT "[ERROR '$plscript_name'] FAILED to CHANGE permission MODES of the OUTPUT DIRECTORY '$outdir' !\n";
		print STDERR "[ERROR '$plscript_name'] $! !\n";
		die ;
	}
     }
     print STDOUT " --- The Output directory '$outdir' is now ready ...\n\n";


    foreach my $ssize (@SSIZES) { # Main outer foreach-loop (over sample sizes) #

	print STDOUT "<<< Sample size = $ssize >>>\n\n";

	my $infile_name = "$infile_pref$ssize$infile_suff";

	# $outfile = "$outdir/$outfile_pref$size$outfile_suff", and

		# Preparation (MOVED UP from below the bottom of the for-loop over SMUs) #

	my $outfile_name = "$outfile_pref$ssize$outfile_suff";
	my $outfile = "$outdir/$outfile_name";

	print STDOUT "Main Output File: $outfile .\n\n";
	##print STDOUT "Output file: $outfile .\n\n";
	
	my $fh_out = open_output_file ($outfile, 1);
	unless (defined $fh_out) {
		print STDOUT "[ERROR '$plscript_name'] FAILED the subroutine 'open_output_file' on the output file '$outfile' !\n";
		print STDOUT "[ERROR '$plscript_name'] $! !\n";
		die ;
	}

		# ADDED in creation of 'make_nulldistri.pimut_cum.pl' (on Jan 29 (Sun), 2012) #

	my $outfile2_name = "$outfile2_pref$ssize$outfile_suff";
	my $outfile2 = "$outdir/$outfile2_name";
	print STDOUT "Auxiliary Output File: $outfile2 .\n\n";

	my $fh_out2 = open_output_file ($outfile2, 1);
	unless (defined $fh_out2) {
		print STDOUT "[ERROR '$plscript_name'] FAILED the subroutine 'open_output_file' on the auxiliary output file '$outfile2' !\n";
		print STDOUT "[ERROR '$plscript_name'] $! !\n";
		die ;
	}

	   ## Initialize the hashes & arrays for the weighted summed weights over SMUs & Snu/SMU's ##
	   ## (ADDED in creation of 'make_nulldistri.pimut_cum.pl' (on Jan 28 (Sat), 2012) #

	my %ctmuthaplos2pimut2smd_wgts;
	    	# = ($ct_mut_haplotypes => {$pimut => \@smd_wgts, ...}, ...), with
		# $smd_wgts[$k] being for the summed weight under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $LABELS_SNU[$k].
	my %ctmuthaplos2smd_tot_wgts;
		# = ($ct_mut_haplotypes => \@smd_tot_wgts, ...), with
		# $smd_tot_wgts[$c] being the summed total weight under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $LABELS_SNU[$k].

	my %pimut2smd_sum_wgts;
		# = ($pimut => \@smd_sum_wgts, ...), with
		# $smd_sum_wgts[$c] being the summation of the summed weights under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $label_snu = $LABELS_SNU[$k].

	my @smd_sum_tot_wgts = (); # $smd_sum_tot_wgts[$c] is the summation of the summation over #{mutant_haplotypes} of total weights over differnt values of pi(mut), under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $label_snu = @LABEL_SNU[$k].
	for (1 .. $CT_SETS_WGT_RATIOS) { push @smd_sum_tot_wgts, 0; }

	print $fh_out2 "\n[1] Overall Statistics & Overall Weights:\n\n";

	for (my $m=0; $m<$CT_SMUs; $m++) { # Main middle foreach-loop (over SMUs) #
	    my $label_smu = $SMUs[$m];
	    my $wgt_smu = $weights_smu->[$m]; ## ADDED in creation of 'make_nulldistri.pimut_cum.pl' (on Jan 28 (Sat), 2012) #
	##foreach my $label_smu (@SMUs) { # Main outer foreach-loop (over SMUs) #

	    print STDOUT "\n<< SMU: $label_smu (weight = $wgt_smu) >>\n\n";
	    print $fh_out2 "\n<< SMU: $label_smu (weight = $wgt_smu) >>\n\n"; # ADDED in creation of 'make_nulldistri.pimut_cum.pl' on Jan 28 (Sat), 2012.

		# Further REVISED in creation of 'summarize_br_pars_msms.3pimut.pl' (on Jan 25 (Wed), 2012)
	    my %ctmuthaplos2pimut2wgts;
	    	# = ($ct_mut_haplotypes => {$pimut => \@wgts, ...}, ...), with
		# $wgts[$k] being for the weight for $label_snu = $LABELS_SNU[$k].
	    my %ctmuthaplos2tot_wgts;
		# = ($ct_mut_haplotypes => \@tot_wgts, ...), with
		# $tot_wgts[$k] being the total weight for $label_snu = $LABELS_SNU[$k].

		## ADDED in creation of 'summarize_br_pars_msms.3pimut.pl' (on Jan 25 (Wed), 2012) #

	    my %pimut2sum_wgts;
		# = ($pimut => \@sum_wgts, ...), with
		# $sum_wgts[$k] being the weight for $label_snu = $LABELS_SNU[$K].

	    my @sum_tot_wgts = (); # $sum_tot_wgts[$k] is the summation over #{mutant_haplotypes} of total weights over differnt values of pi(mut).
	    for (1 .. $CT_SNUS) { push @sum_tot_wgts, 0; }

	    my @cts_reps = my @cts_fxd_wt = my @cts_fxd_mut = ();
	    my @cts_undef_pimut = (); # ADDED in revision of 'summarize_br_pars_msms.3pimut.pl' on Jan 25 (Wed), 2012. #

	    for (my $k=0; $k< $CT_SNUS; $k++) { # Main inner for-loop  (over SNUs) #

		my $label_snu = $LABELS_SNU[$k];
		print STDOUT "\n< Snu: $label_snu >\n\n";

	# $infile = "$indir/$infile_pref$size$infile_suff", with
	# $indir = "$intopdir/$indir_pref1$label_snu/$indir_pref2$label_smu/$indir_pref3$label_sAA".

		my $indir = "$intopdir/$indir_pref1$label_snu/$indir_pref2$label_smu/$indir_pref3$label_sAA";

		unless (( -d $indir ) and ( -r $indir )) {
		    print STDOUT "[ERROR '$plscript_name'] CanNOT FIND a readable INPUT DIRECTORY '$indir' !\n";
		    print STDERR "[ERROR '$plscript_name'] $! !\n";
		    die ;
		}
		print STDOUT "Input directory: $indir .\n\n";


			# First, read in an input file #

		my $infile = "$indir/$infile_name";
		unless (( -f $infile ) and ( -r $infile )) {
		    print STDOUT "[ERROR '$plscript_name'] CanNOT FIND a readable INPUT FILE '$infile' !\n";
		    print STDERR "[ERROR '$plscript_name'] $! !\n";
		    die ;
		}
	
		print STDOUT "Input file: $infile .\n";
	
		local $/ = "\n"; # Use newline characters  as input record searators #

		my $fh_in = open_input_file2 ($infile); 
		unless (defined $fh_in) {
		    print STDOUT "[ERROR '$plscript_name'] FAILED the subroutine 'open_input_file2' on the input file '$infile' !\n";
		    print STDERR "[ERROR '$plscript_name'] $! !\n";
		    die ;
		}

		my ($ct_reps, $ct_fxd_wt, $ct_fxd_mut);

		while (my $in = <$fh_in>) { # Preliminary while-loop (to process the header) #
		    ($in =~ s/^\#\s*//) or last;
		    if ($in =~ s/^\#\{Simulated_replicates\}\s*\=\s*//) {
			($ct_reps) = ($in =~ m/^(\d+)/);
		    } elsif ($in =~ s/^\#\{Cases\s+where\s+the\s+WT\s+was\s+fixed\}\s*\=\s*//) {
			($ct_fxd_wt) = ($in =~ m/^(\d+)/); 
		    } elsif ($in =~ s/^\#\{Cases\s+where\s+the\s+mutant\s+was\s+fixed\}\s*\=\s*//) {
			($ct_fxd_mut) = ($in =~ m/^(\d+)/);
		    }
		}		# End of the preliminary while-loop (to process the header) #
		unless ((defined $ct_reps) and (defined $ct_fxd_wt) and (defined $ct_fxd_mut)) {
		    foreach my $rf (\$ct_reps, \$ct_fxd_wt, \$ct_fxd_mut) { (defined ${$rf}) or (${$rf} = 'undefined'); }
		     print STDOUT "[ERROR '$plscript_name'] ILLEGAL INPUT: Some of the parameters canNOT be EXTRACTED from the header: (\$ct_reps, \$ct_fxd_wt, \$ct_fxd_mut) = ($ct_reps, $ct_fxd_wt, $ct_fxd_mut) !\n";
		    print STDERR "[ERROR '$plscript_name'] $! !\n";
		    die ;
		}

		$cts_reps[$k] = $ct_reps;
		$cts_fxd_wt[$k] = $ct_fxd_wt;
		$cts_fxd_mut[$k] = $ct_fxd_mut;

		my $counter = 0; ### For TESTING the script
		my $ct_undef_pimut = 0;
		while (my $in = <$fh_in>) { # Main while-loop (over entries with weights assigned) #

		# NOTE in creation of 'make_nulldistri.pimut_cond_ctfwmuts_cum.pl' on Feb 24 (Fri), 2012:
		# Throughout this script, the labeling 'ct_muthaplos'
		# actually refers to 'avct_fwd_mutations'.

		    my ($ct_muthaplos, $ct_mutants, $pimut, $wgt) = split /\s+/, $in;
		    (defined $wgt) or next;

		    $counter++;
		    ### ($counter>5) and last; ### For TESTING the script ###

			# Skip if pi(muts) is NOT defined.
			# (ADDED in revision of 'summarize_br_pars_msms.3pimut.pl' on Jan 25 (Wed), 2012) #
		    if ($pimut eq 'undef') {
			$ct_undef_pimut++;
			next;
		    }

		    my $pimut2wgts = $ctmuthaplos2pimut2wgts{$ct_muthaplos};
		    (defined $pimut2wgts) or ($pimut2wgts = $ctmuthaplos2pimut2wgts{$ct_muthaplos} = {});
		    my $wgts = $pimut2wgts->{$pimut};


			# A BUG Fixed on Dec 31 (Sat), 2011 #
		    unless (defined $wgts) {
		    ##unless (defined $totct_muts2wgts) {
			my @wgts = ();

			for (1 .. $CT_SNUS) { push @wgts, 0; }

		 	$wgts = $pimut2wgts->{$pimut} = \@wgts;
		    }
		    $wgts->[$k] += $wgt;

		    my $tot_wgts = $ctmuthaplos2tot_wgts{$ct_muthaplos};
		    unless (defined $tot_wgts) {
			my @tot_wgts = ();

			for (1 .. $CT_SNUS) { push @tot_wgts, 0; }

			$tot_wgts = $ctmuthaplos2tot_wgts{$ct_muthaplos} = \@tot_wgts ;
		    }
		    $tot_wgts->[$k] += $wgt;

	    # my %ctmuthaplos2tot_wgts;
		# = ($ct_mut_haplotypes => \@tot_wgts, ...), with
		# $tot_wgts[$k] being the total weight for $label_snu = $LABELS_SNU[$k].

			# Calculate the weight assigned to each value of pi(muts), summed over different #{mutant_haplotypes}.
			# (ADDED in creation of 'summarize_br_pars_msms.3pimut.pl' on Jan 25 (Wed), 2012.)

		    my $sum_wgts = $pimut2sum_wgts{$pimut};
		    unless (defined $sum_wgts) {
			my @sum_wgts = ();
			for (1 .. $CT_SNUS) { push @sum_wgts, 0; }
			$sum_wgts = $pimut2sum_wgts{$pimut} = \@sum_wgts;
		    }
		    $sum_wgts->[$k] += $wgt;

	    # my %pimut2sum_wgts;
		# = ($pimut => \@sum_wgts, ...), with
		# $sum_wgts[$k] being the weight for $label_snu = $LABELS_SNU[$K].

		    $sum_tot_wgts[$k] += $wgt;

	    # my @sum_tot_wgts = (); # $sum_tot_wgts[$k] is the summation over #{mutant_haplotypes} of total weights over differnt values of pi(mut).

		}		# End of the main while-loop (over entries with weights assigned) #

		close ($fh_in);

			# ADDED in revision of 'summarize_br_pars_msms.3pimut.pl' (on Jan 25 (Wed), 2012) #
		$cts_undef_pimut[$k] = $ct_undef_pimut;


		print STDOUT "\n --- Snu = $label_snu Ended ---\n\n";
	    }		# End of the main inner foreach-loop (over SNUs) #

			## Output the summary results ##

=MOVED UP above the top of the foreach-loop over SNUs, in creation of 'make_nulldistri.pimut_cum.pl'.
			# Preparation #

	    my $outfile_name = "$outfile_pref$ssize$outfile_suff";
	    my $outfile = "$outdir/$outfile_name";

		print STDOUT "Output file: $outfile .\n\n";
	
	    my $fh_out = open_output_file ($outfile, 1);
	    unless (defined $fh_out) {
		print STDOUT "[ERROR '$plscript_name'] FAILED the subroutine 'open_output_file' on the output file '$outfile' !\n";
		print STDOUT "[ERROR '$plscript_name'] $! !\n";
		die ;
	    }
=cut

	    print $fh_out2 "A. Overall Statistics:\n\n";
	    ##print $fh_out "[1] Overall Statistics:\n\n";

	    print $fh_out2 join ("\t", 'Count\label_Snu', @LABELS_SNU), "\n";

	    print $fh_out2 join ("\t", '#{Simulated_Replicates}', @cts_reps), "\n";
	    print $fh_out2 join ("\t", '#{Cases_with_fixed_WT}', @cts_fxd_wt), "\n";
	    print $fh_out2 join ("\t", '#{Cases_with_fixed_mutants}', @cts_fxd_mut), "\n";
	    ## my @cts_reps = my @cts_fxd_wt = my @cts_fxd_mut = ();

	    print $fh_out2 join ("\t", '#{Cases_with_undefined_pi(muts)}', @cts_undef_pimut), "\n";
	    print $fh_out2 "\n\n";


		# MAJOR REVISION in creation of 'make_nulldistri.pimut_cond_ctfwmuts_cum.pl' on Feb 24 (Fri), 2012. #
	    print $fh_out2 "B. RAW Total weights for specific Snu's and for specific Int(Av#{forward_mutations}) (BEFORE divided by Total#{replicates with the respective Snu}):\n\n";
	    print $fh_out2 join ("\t", 'Int(Av#{fwd_mutations})\label_Snu', @LABELS_SNU), "\n";
=for the predecessor 'make_nulldistri.pimut_cond_ctmuthaplos_cum.pl'
	    print $fh_out2 "B. RAW Total weights for specific Snu's and for specific #{mutant_haplotypes} (BEFORE divided by Total#{replicates with the respective Snu}):\n\n";
	    ##print $fh_out "[2] Total weights for specific Snu's and for specific #{mutant_haplotypes}:\n\n";
	    print $fh_out2 join ("\t", '#{mutant_haplotypes}\label_Snu', @LABELS_SNU), "\n";
=cut
	    foreach my $ctmuthaplos (sort {$a <=> $b} keys %ctmuthaplos2tot_wgts ) { # 2nd inner foreach-loop (over #{mutant_haplotypes}) #
		my $tot_wgts = $ctmuthaplos2tot_wgts{$ctmuthaplos};
		print $fh_out2 join ("\t", $ctmuthaplos, @{$tot_wgts}), "\n";
		##print $fh_out join ("\t", $ctmuthaplos, @{$tot_wgts}), "\n";
	    }		# ERnd of the 2nd inner foreach-loop (over #{mutant_haplotypes}) #
	    print $fh_out2 "\n";

	    # my %ctmuthaplos2tot_wgts;
		# = ($ct_mut_haplotypes => \@tot_wgts, ...), with
		# $tot_wgts[$k] being the total weight for $label_snu = $LABELS_SNU[$k].

		# ADDED in creation of 'summarize_br_pars_msms.3pimut.pl' on Jan 25 (Wed), 2012. #
	    print $fh_out2 join ("\t", 'Total', @sum_tot_wgts) , "\n";
	    print $fh_out2 "\n\n";


=Set aside for DEBUGING the script (1)
		## Record the overall distribution of pi(muts). ##
		## (ADDED as a MAJOR REVISION in creation of 'summarize_br_pars_msms.3pimut.pl' on Jan 25 (Wed), 2012.)
	    print $fh_out "\nC. Cumulative frequencies for specific values of pi(muts), summed over different #{mutant_haplotypes}, and conditional on Snu's:\n\n";
	    ##print $fh_out "\n[3] Cumulative frequencies for specific values of pi(muts), summed over different #{mutant_haplotypes}, and conditional on Snu's:\n\n";

		# ADDED in creation of 'summarize_br_pars_msms.3pimut.pl' on Jan 25 (Wed), 2012. #
	    my @abs_cum_sum_wgts = ();
	    for (1 .. $CT_SNUS) { push @abs_cum_sum_wgts, 0; }

	    print $fh_out join ("\t", 'pi(muts)\label_sNU', @LABELS_SNU), "\n";
=cut
	    foreach my $pimut (sort {$b <=> $a} keys %pimut2sum_wgts) { # 3rd inner foreach-loop (over the values of pi(muts)) #
	    ##foreach my $pimut (sort {$a <=> $b} keys %pimut2sum_wgts) { # 3rd inner foreach-loop (over the values of pi(muts)) #
		my $sum_wgts = $pimut2sum_wgts{$pimut};

		my $ct_ok1 = 0;
		foreach my $stwgt (@sum_tot_wgts) { ($stwgt >= $LB_SMD_WGT) and ($ct_ok1++); }
		($ct_ok1>0) or next; # Skip if no ingredient satisfies the condition.

		## ADDED in creation of 'make_nulldistri.pimut_cum.pl' on Jan 28 (Sat), 2012. #
		my @incr_ssm_wgts = ();
		my $ct_ok2=0;
		for (my $s=0; $s<$CT_SETS_WGT_RATIOS; $s++) { # 1st inner-inner for-loop (over the combinations of Snu/Smu's) #
		   my $incr = 0;
		   my $set_wgt_rts = $SETS_WEIGHTS_RATIOS[$s];
		   for (my $k=0; $k<$CT_SNUS; $k++) {
			($sum_tot_wgts[$k]>=$LB_SMD_WGT) or next;
			$incr += $set_wgt_rts->[$k] * $sum_wgts->[$k]/$cts_reps[$k];
		   }
		   $incr_ssm_wgts[$s] = $incr;
		   ($incr > 0) and ($ct_ok2++);
		}		# End of the 1st inner-inner for-loop (over the combinations of Snu/Smu's) #
		($ct_ok2>0) or next; # Skip if there is no increments for any combination.

		my $smd_sum_wgts = $pimut2smd_sum_wgts{$pimut};
		unless (defined $smd_sum_wgts) {
		    my @tmp_ssm_wgts = ();
		    for (1 .. $CT_SETS_WGT_RATIOS) { push @tmp_ssm_wgts, 0; } 
		    $smd_sum_wgts = $pimut2smd_sum_wgts{$pimut} = \@tmp_ssm_wgts;
		}

		for (my $s=0; $s<$CT_SETS_WGT_RATIOS; $s++) { # 2nd inner-inner for-loop (over the combinations of Snu/Smu's) #
		    my $incr = $wgt_smu * $incr_ssm_wgts[$s];
		    $smd_sum_wgts->[$s] += $incr;
		    $smd_sum_tot_wgts[$s] += $incr;
		}	# End of the 2nd inner-inner for-loop (over the combinations of Snu/Smu's) #

	# my %pimut2smd_sum_wgts;
		# = ($pimut => \@smd_sum_wgts, ...), with
		# $smd_sum_wgts[$c] being the summation of the summed weights under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $label_snu = $LABELS_SNU[$k].

	# my @smd_sum_tot_wgts = (); # $smd_sum_tot_wgts[$c] is the summation of the summation over #{mutant_haplotypes} of total weights over differnt values of pi(mut), under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $label_snu = @LABEL_SNU[$k].

=Set aside for DEBUGING the script (2)
		  # MAJOR REVISION in creation of 'summarize_br_pars_msms.3pimut.pl' (on Jan 25 (Wed), 2012). #
		my @rel_cum_sum_wgts = ();
		for (my $k=0; $k<$CT_SNUS; $k++) { # Innermost for-loop (over sAAs) #

		    my $abs_cum_sum_wgt = $abs_cum_sum_wgts[$k] += $sum_wgts->[$k];
		    my $tot_sum_wgt = $sum_tot_wgts[$k];
	    ## my @sum_tot_wgts = (); # $sum_tot_wgts[$k] is the summation over #{mutant samples} of total weights over different values of Tajima's D. #
		    (defined $tot_sum_wgt) or ($tot_sum_wgt = 0);
		    $rel_cum_sum_wgts[$k] = ($tot_sum_wgt>0) ? $abs_cum_sum_wgt/$tot_sum_wgt : 'NA' ;
		}	# End of the innermost for-loop (over sAAs) #
		print $fh_out join ("\t", $pimut, @rel_cum_sum_wgts), "\n";
		##print $fh_out join ("\t", $pimut, @{$sum_wgts}), "\n";
=cut
	    }	# End of the 3rd inner foreach-loop (over the values of pi(muts)) #

=Set aside for DEBUGING the script (3)
	    print $fh_out "\n\n";
=cut


=Set aside for DEBUGING the script (4)
		# Further MODIFIED in creation of 'summarize_br_pars_msms.3pimut_cum.pl' on Jan 25 (Wed), 2012. #
	    print $fh_out "\nD. Cumulative frequencies for specific values of pi(muts), conditional on #{mutant_haplotypes} and Snu's:\n\n";
	    ##print $fh_out "\n[4] Cumulative frequencies for specific values of pi(muts), conditional on #{mutant_haplotypes} and Snu's:\n\n";
	    ##	# MODIFIED in creation of 'summarize_br_pars_msms.3pimut.pl' on Jan 25 (Wed), 2012. #
	    ##print $fh_out "\n[4] Weights for specific values of pi(muts), conditional on #{mutant_haplotypes} and Snu's:\n\n";
=cut

	    foreach my $ct_muthaplos (sort {$a <=> $b} keys %ctmuthaplos2pimut2wgts) { # 3rd inner foreach-loop (over #{mutant_haplotypes}) #

=Set aside for DEBUGING the script (5)
		print $fh_out "\n< #{mutant_haplotypes} = $ct_muthaplos >\n\n";

		print $fh_out join ("\t", 'pi(muts)\label_Snu', @LABELS_SNU), "\n";
=cut

		my $pimut2wgts = $ctmuthaplos2pimut2wgts{$ct_muthaplos};


		my $tot_wgts = $ctmuthaplos2tot_wgts{$ct_muthaplos};
		my @abs_cum_wgts = ();
		for (1 .. $CT_SNUS) { push @abs_cum_wgts, 0; }

		my $ct_oks1 = 0;
		foreach my $twgt (@{$tot_wgts}) { ($twgt >= $LB_SMD_WGT) and ($ct_oks1++); }
		($ct_oks1>0) or next; # Skip if no ingredient satisfies the condition.

		 # Make a weighted sum (over Snu/Smu's) of total weights (over pi(muts)) #
		 # (ADDED in creation of 'make_nulldistri.pimut_cum.pl' on Jan 28 (Sat), 2012.) #
		my @incr_stot_wgts = ();
		my $ct_oks2=0;
		for (my $c=0; $c<$CT_SETS_WGT_RATIOS; $c++) { # 1st inner-inner for-loop (over the combinations of the weights) #
		    my $set_wgt_ratios = $SETS_WEIGHTS_RATIOS[$c];
		    my $incr=0;
		    for (my $k=0; $k<$CT_SNUS; $k++) {
			my $tot_wgt = $tot_wgts->[$k];
			($tot_wgt>=$LB_SMD_WGT) or next;
			$incr += $set_wgt_ratios->[$k] * $tot_wgt/$cts_reps[$k];
		    }
		    ($incr>0) and ($ct_oks2++);
		    $incr_stot_wgts[$c] = $incr;
		}	# End of the 1st inner-inner for-loop (over the combinations of the weights) #
		($ct_oks2>0) or next; # Skip if there is no chance of increment. #

		my $smd_tot_wgts = $ctmuthaplos2smd_tot_wgts{$ct_muthaplos};
		unless (defined $smd_tot_wgts) {
		    my @tmp_stot_wgts = ();
		    for (1 .. $CT_SETS_WGT_RATIOS) { push @tmp_stot_wgts, 0; }
		    $smd_tot_wgts = $ctmuthaplos2smd_tot_wgts{$ct_muthaplos} = \@tmp_stot_wgts;
		}

		for (my $c=0; $c<$CT_SETS_WGT_RATIOS; $c++) { $smd_tot_wgts->[$c] += $wgt_smu * $incr_stot_wgts[$c]; }

		my $pimut2smd_wgts = $ctmuthaplos2pimut2smd_wgts{$ct_muthaplos};
		(defined $pimut2smd_wgts) or ($pimut2smd_wgts = $ctmuthaplos2pimut2smd_wgts{$ct_muthaplos} = {});


		foreach my $pimut (sort {$b <=> $a} keys %{$pimut2wgts}) { # Inner-inner foreach-loop (over pi among mutant samples) #
		##foreach my $pimut (sort {$a <=> $b} keys %{$pimut2wgts}) { # Inner-inner foreach-loop (over pi among mutant samples) #
		    my $wgts = $pimut2wgts->{$pimut};


			# Make a weighted sum (over Snu/Smu's) of weights #
			# (ADDED in creation of 'make_nulldistri.pimut_cum.pl' on Jan 28 (Sat), 2012.) #
		    my @incr_wgts3 = ();
		    my $ct_oks3=0;
		    for (my $c=0; $c<$CT_SETS_WGT_RATIOS; $c++) { # 1st inner-inner for-loop (over the combinations of Snu/Smu's) #
			my $set_wgt_ratios = $SETS_WEIGHTS_RATIOS[$c];
			my $incr=0;
			for (my $k=0; $k<$CT_SNUS; $k++) {
			    my $tot_wgt = $tot_wgts->[$k];
			    ($tot_wgt>=$LB_SMD_WGT) or next;
			    $incr += $set_wgt_ratios->[$k] * $wgts->[$k]/$cts_reps[$k];
			}
			($incr>0) and ($ct_oks3++);
			$incr_wgts3[$c] = $incr;
		    }	# End of the 1st inner-inner for-loop (over the combinations of the Snu/Smu's) #
		    ($ct_oks3>0) or next; # Skip if there is no increment for any combination. #

		    my $smd_wgts = $pimut2smd_wgts->{$pimut};
		    unless (defined $smd_wgts) {
			my @tmp_smd_wgts = ();
			for (1 .. $CT_SETS_WGT_RATIOS) { push @tmp_smd_wgts, 0; }
		 	$smd_wgts = $pimut2smd_wgts->{$pimut} = \@tmp_smd_wgts;
		    }

		    for (my $c=0; $c<$CT_SETS_WGT_RATIOS; $c++) { $smd_wgts->[$c] += $wgt_smu * $incr_wgts3[$c]; }

	# my %ctmuthaplos2pimut2smd_wgts;
	    	# = ($ct_mut_haplotypes => {$pimut => \@smd_wgts, ...}, ...), with
		# $smd_wgts[$k] being for the summed weight under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $LABELS_SNU[$k].

=for DEBUGING the script (6)
			## ADDED in creation of 'summarize_br_pars_msms.2max_cum.pl' (on Jan 1 (Sun), 2012) #
		    my @rel_cum_wgts = ();
		    for (my $k=0; $k<$CT_SNUS; $k++) { # Innermost for-loop (over sAAs) #

			my $abs_cum_wgt = $abs_cum_wgts[$k] += $wgts->[$k];
			my $tot_wgt = $tot_wgts->[$k];
			(defined $tot_wgt) or ($tot_wgt = 0);
			$rel_cum_wgts[$k] = ($tot_wgt>0) ? $abs_cum_wgt/$tot_wgt : 'NA' ;
		    }		# End of the innermost for-loop (over sAAs) #

			# MAJOR REVISION in creation of 'summarize_br_pars_msms.3pimut.pl' (on Jan 25 (Wed), 2012). #
			# => Further REVISION in creation of 'summarize_br_pars_msms.3pimut_cum.pl' (on Jan 25 (Wed), 2012).
		    print $fh_out join ("\t", $pimut, @rel_cum_wgts), "\n";
		    ##print $fh_out join ("\t", $pimut, @{$wgts}), "\n";
=cut


		}		# End of the inner-iner foreach-loop (over the values of pi(muts)) #

	    }		# End of the 3rd inner foreach-loop (over #{mutant_haplotypes}) #
	    # my %ctmuthaplos2frqcommonestmut2wgts;
		# = ($ct_mut_haplotypes => {$freq_commonest_mut_haplotype => \@wgts, ...}, ...), with
		# $wgts[$k] being for the weight for $label_snu = $LABELS_SNU[$k].

	    print STDOUT "\n  === SMU = $label_smu ENDED ===\n\n";


	}		# End of the main middle for-loop (over SMUs) #
	##}		# End of the main outer foreach-loop (over SMUs) #

		## Output the Null Distributions ##
		## (ADDED in creation of 'make_nulldistri.pimut_cum.pl' on Jan 29 (Sun), 2012) ## 

	   # MAJOR REVISION in creation of 'make_nulldistri.pimut_cond_ctfwmuts_cum.pl' on Feb 24 (Fri), 2012. #
	print $fh_out2 "[2] Total weights for specific Combinations of Snu/Smu's and for specific Int(Av#{fwd_mutations}):\n\n";
	print $fh_out2 join ("\t", 'Int(Av#{fwd_mutations})\Combination', @NAMES_SETS_WEIGHTS_RATIOS), "\n";
=for the predecessor 'make_nulldistri.pimut_cond_ctmuthaplos_cum.pl'
	print $fh_out2 "[2] Total weights for specific Combinations of Snu/Smu's and for specific #{mutant_haplotypes}:\n\n";
	print $fh_out2 join ("\t", '#{mutant_haplotypes}\Combination', @NAMES_SETS_WEIGHTS_RATIOS), "\n";
	##print $fh_out join ("\t", '#{mutant_haplotypes}\label_Snu', @LABELS_SNU), "\n";
=cut
	foreach my $ctmuthaplos (sort {$a <=> $b} keys %ctmuthaplos2smd_tot_wgts ) { # 2nd inner foreach-loop (over #{mutant_haplotypes}) #
	##foreach my $ctmuthaplos (sort {$a <=> $b} keys %ctmuthaplos2tot_wgts ) { # 2nd inner foreach-loop (over #{mutant_haplotypes}) #
	    my $smd_tot_wgts = $ctmuthaplos2smd_tot_wgts{$ctmuthaplos};
	    ##my $tot_wgts = $ctmuthaplos2tot_wgts{$ctmuthaplos};
	    print $fh_out2 join ("\t", $ctmuthaplos, @{$smd_tot_wgts}), "\n";
	    ##print $fh_out join ("\t", $ctmuthaplos, @{$tot_wgts}), "\n";
	}		# ERnd of the 2nd inner foreach-loop (over #{mutant_haplotypes}) #
	print $fh_out2 "\n";

	# my %ctmuthaplos2smd_tot_wgts;
		# = ($ct_mut_haplotypes => \@smd_tot_wgts, ...), with
		# $smd_tot_wgts[$c] being the summed total weight under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $LABELS_SNU[$k].

		# ADDED in creation of 'summarize_br_pars_msms.3pimut.pl' on Jan 25 (Wed), 2012. #
	##print $fh_out2 join ("\t", 'Total', @smd_sum_tot_wgts) , "\n";
	####print $fh_out join ("\t", 'Total', @sum_tot_wgts) , "\n";
	##print $fh_out2 "\n\n";
	  # => MOVED to the bottom of [3] on Jan 29 (Sun), 2012. #

	# my @smd_sum_tot_wgts = (); # $smd_sum_tot_wgts[$c] is the summation of the summation over #{mutant_haplotypes} of total weights over differnt values of pi(mut), under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $label_snu = @LABEL_SNU[$k].

	  # MAJOR REVISION in creation of 'make_nulldistri.pimut_cond_ctfwmuts_cum.pl' on Feb 24 (Fri), 2012. #
	print $fh_out "\n[1] Cumulative frequencies for specific values of pi(muts) and for specific combinations of Snu/Smu's, summed over different Int(Av#{fwd_mutations}):\n\n";
	print $fh_out2 "\n[3] Weights for specific values of pi(muts) and for specific combinations of Snu/Smu's, summed over different Int(Av#{fwd_mutations}):\n\n";
=for the predecessor 'make_nulldistri.pimut_cond_ctmuthaplos_cum.pl'
	print $fh_out "\n[1] Cumulative frequencies for specific values of pi(muts) and for specific combinations of Snu/Smu's, summed over different #{mutant_haplotypes}:\n\n";
	print $fh_out2 "\n[3] Weights for specific values of pi(muts) and for specific combinations of Snu/Smu's, summed over different #{mutant_haplotypes}:\n\n";
=cut

	my @abs_cum_smd_sum_wgts = ();
	for (1 .. $CT_SETS_WGT_RATIOS) { push @abs_cum_smd_sum_wgts, 0; }
	##my @abs_cum_sum_wgts = ();
	##for (1 .. $CT_SNUS) { push @abs_cum_sum_wgts, 0; }

	print $fh_out join ("\t", 'pi(muts)\Combination', @NAMES_SETS_WEIGHTS_RATIOS), "\n";
	print $fh_out2 join ("\t", 'pi(muts)\Combination', @NAMES_SETS_WEIGHTS_RATIOS), "\n";
	##print $fh_out join ("\t", 'pi(muts)\label_sNU', @LABELS_SNU), "\n";

	foreach my $pimut (sort {$b <=> $a} keys %pimut2smd_sum_wgts) { # 3rd inner foreach-loop (over the values of pi(muts)) #
	##foreach my $pimut (sort {$a <=> $b} keys %pimut2smd_sum_wgts) { # 3rd inner foreach-loop (over the values of pi(muts)) #

			### RESTART FROM HERE ###

		my $smd_sum_wgts = $pimut2smd_sum_wgts{$pimut};

		my @rel_cum_smd_sum_wgts = ();
		for (my $c=0; $c<$CT_SETS_WGT_RATIOS; $c++) { # Innermost for-loop (over combinations of Snu/Smu's) #
		##for (my $k=0; $k<$CT_SNUS; $k++) { # Innermost for-loop (over sAAs) #

		    my $abs_cum_smd_sum_wgt = $abs_cum_smd_sum_wgts[$c] += $smd_sum_wgts->[$c];
		    my $smd_sum_tot_wgt = $smd_sum_tot_wgts[$c];
		    ##my $tot_sum_wgt = $sum_tot_wgts[$k];
	    ## my @sum_tot_wgts = (); # $sum_tot_wgts[$k] is the summation over #{mutant samples} of total weights over different values of Tajima's D. #
		    (defined $smd_sum_tot_wgt) or ($smd_sum_tot_wgt = 0);
		    $rel_cum_smd_sum_wgts[$c] = ($smd_sum_tot_wgt>0) ? $abs_cum_smd_sum_wgt/$smd_sum_tot_wgt : 'NA' ;
		    ##(defined $tot_sum_wgt) or ($tot_sum_wgt = 0);
		    ##$rel_cum_sum_wgts[$k] = ($tot_sum_wgt>0) ? $abs_cum_sum_wgt/$tot_sum_wgt : 'NA' ;
		}	# End of the innermost for-loop (over combinations of Snu/Smu's) #

		print $fh_out join ("\t", $pimut, @rel_cum_smd_sum_wgts), "\n";
		##print $fh_out join ("\t", $pimut, @rel_cum_sum_wgts), "\n";

		print $fh_out2 join ("\t", $pimut, @{$smd_sum_wgts}), "\n";
		##print $fh_out join ("\t", $pimut, @{$sum_wgts}), "\n";

	}	# End of the 3rd inner foreach-loop (over the values of pi(muts)) #

	print $fh_out "\n\n";

		# MOVED from the bottom of [2] on Jan 29 (Sun), 2012. #
	print $fh_out2 "\n";
	print $fh_out2 join ("\t", 'Total', @smd_sum_tot_wgts) , "\n";

	print $fh_out2 "\n\n";

	# my %pimut2smd_sum_wgts;
		# = ($pimut => \@smd_sum_wgts, ...), with
		# $smd_sum_wgts[$c] being the summation of the summed weights under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $label_snu = $LABELS_SNU[$k].

	   # MAJOR REVISION in creation of 'make_nulldistri.pimut_cond_ctfwmuts_cum.pl' on Feb 24 (Fri), 2012. #
	print $fh_out "\n[2] Cumulative frequencies for specific values of pi(muts), conditional on Int(Av#{fwd_mutations}) and specific combinations of Snu/Smu's:\n\n";
	print $fh_out2 "\n[4] Summed weights for specific values of pi(muts), conditional on Int(Av#{fwd_mutations}) and specific combinations of Snu/Smu's:\n\n";
=for the predecessor 'make_nulldistri.pimut_cond_ctmuthaplos_cum.pl'
	print $fh_out "\n[2] Cumulative frequencies for specific values of pi(muts), conditional on #{mutant_haplotypes} and specific combinations of Snu/Smu's:\n\n";
	print $fh_out2 "\n[4] Summed weights for specific values of pi(muts), conditional on #{mutant_haplotypes} and specific combinations of Snu/Smu's:\n\n";
=cut

	foreach my $ct_muthaplos (sort {$a <=> $b} keys %ctmuthaplos2pimut2smd_wgts) { # 3rd inner foreach-loop (over #{mutant_haplotypes}) #
	##foreach my $ct_muthaplos (sort {$a <=> $b} keys %ctmuthaplos2pimut2wgts) { # 3rd inner foreach-loop (over #{mutant_haplotypes}) #

	   # MAJOR REVISION in creation of 'make_nulldistri.pimut_cond_ctfwmuts_cum.pl' on Feb 24 (Fri), 2012. #
		print $fh_out "\n< Int(Av#{fwd_mutations}) = $ct_muthaplos >\n\n";
		print $fh_out2 "\n< Int(Av#{fwd_mutations}) = $ct_muthaplos >\n\n";
=for the predecessor 'make_nulldistri.pimut_cond_ctmuthaplos_cum.pl'
		print $fh_out "\n< #{mutant_haplotypes} = $ct_muthaplos >\n\n";
		print $fh_out2 "\n< #{mutant_haplotypes} = $ct_muthaplos >\n\n";
=cut

		print $fh_out join ("\t", 'pi(muts)\Combination', @NAMES_SETS_WEIGHTS_RATIOS), "\n";
		print $fh_out2 join ("\t", 'pi(muts)\Combination', @NAMES_SETS_WEIGHTS_RATIOS), "\n";

		my $pimut2smd_wgts = $ctmuthaplos2pimut2smd_wgts{$ct_muthaplos};
		##my $pimut2wgts = $ctmuthaplos2pimut2wgts{$ct_muthaplos};

		my $smd_tot_wgts = $ctmuthaplos2smd_tot_wgts{$ct_muthaplos};
	# my %ctmuthaplos2smd_tot_wgts;
		##my $tot_wgts = $ctmuthaplos2tot_wgts{$ct_muthaplos};

		my @abs_cum_smd_wgts = ();
		for (1 .. $CT_SETS_WGT_RATIOS) { push @abs_cum_smd_wgts, 0; }
		##my @abs_cum_wgts = ();
		##for (1 .. $CT_SNUS) { push @abs_cum_wgts, 0; }

		foreach my $pimut (sort {$b <=> $a} keys %{$pimut2smd_wgts}) { # Inner-inner foreach-loop (over pi among mutant samples) #
		##foreach my $pimut (sort {$a <=> $b} keys %{$pimut2smd_wgts}) { # Inner-inner foreach-loop (over pi among mutant samples) #

		    my $smd_wgts = $pimut2smd_wgts->{$pimut};
		    ##my $wgts = $pimut2wgts->{$pimut};

		    my @rel_cum_smd_wgts = ();
		    for (my $c=0; $c<$CT_SETS_WGT_RATIOS; $c++) { # Innermost for-loop (over combinations of Snu/Smu's) #
		    ##for (my $k=0; $k<$CT_SNUS; $k++) { # Innermost for-loop (over sAAs) #

			my $abs_cum_smd_wgt = $abs_cum_smd_wgts[$c] += $smd_wgts->[$c];
			my $smd_tot_wgt = $smd_tot_wgts->[$c];
			(defined $smd_tot_wgt) or ($smd_tot_wgt = 0);
			$rel_cum_smd_wgts[$c] = ($smd_tot_wgt>0) ? $abs_cum_smd_wgt/$smd_tot_wgt : 'NA' ;
			##my $abs_cum_wgt = $abs_cum_wgts[$k] += $wgts->[$k];
			##my $tot_wgt = $tot_wgts->[$k];
			##(defined $tot_wgt) or ($tot_wgt = 0);
			##$rel_cum_wgts[$k] = ($tot_wgt>0) ? $abs_cum_wgt/$tot_wgt : 'NA' ;

		    }		# End of the innermost for-loop (over combinations of Snu/Smu's) #

		    print $fh_out join ("\t", $pimut, @rel_cum_smd_wgts), "\n";
		    ##print $fh_out join ("\t", $pimut, @rel_cum_wgts), "\n";
		    print $fh_out2 join ("\t", $pimut, @{$smd_wgts}), "\n";
		    ##print $fh_out join ("\t", $pimut, @{$wgts}), "\n";

		}		# End of the inner-iner foreach-loop (over the values of pi(muts)) #

		print $fh_out "\n";
		print $fh_out2 "\n";

	}		# End of the 3rd inner foreach-loop (over #{mutant_haplotypes}) #

	# my %ctmuthaplos2pimut2smd_wgts;
	    	# = ($ct_mut_haplotypes => {$pimut => \@smd_wgts, ...}, ...), with
		# $smd_wgts[$k] being for the summed weight under the $c th weight set, @wgts_ratios = @{$SETS_WEIGHTS_RATIOS[$c]}, where $wgts_ratios[$k] is the weight for $LABELS_SNU[$k].

##our @SETS_WEIGHTS_RATIOS = (
##	[1, 1, 1, 1, 1, 0],
##	[1, 0, 0, 0, 0, 0],
##	[0, 0, 1, 0, 0, 0],
##	[0, 0, 0, 0, 1, 0],
##	[0, 0, 0, 0, 0, 1]
##	);
##our @NAMES_SETS_WEIGHTS_RATIOS = ('Even', @LABELS_SNU[0, 2, 4, 5]);
##our $CT_SETS_WGT_RATIOS = @SETS_WEIGHTS_RATIOS;

	close ($fh_out);
	close ($fh_out2);

	print STDOUT "\n ... Sample size = $ssize ended ...\n\n";

    }		# End of the main outer foreach-loop (over sample sizes) #

    print STDOUT "\n ||| sAA = $label_sAA ENDED !! |||\n\n";

}		# End of the Main outermost foreach-loop (over sAA's) #


print STDOUT "\n\n##### ALL THE PROCESSED ENDED !!! #####\n\n\n";



	### DEFINITIONS OF THE SUBROUTINES ###

# make_weights_smu:
#	This subroutine makes weight factors to be assigned to the values of Smu.
#	The weight factor is based on the power-law:
#	Prb [ Smu >= x] = A * x**{-alpha}.
#
# INPUT: (\@smus, $smu_min, $smu_max, $alpha), where
#	@smus are the values of Smu used for the simulations;
#	$smu_min is the minimum value of Smu;
#	$smu_max is the maximum value of Smu;
#	$alpha is the power for the weight factor.
# 
# OUTPUT: \@weights, where
#	$weights[$k] is the weight factor assigned to $smu[$k].
#
#
sub make_weights_smu (\@$$$) {

	my ($smus, $smu_min, $smu_max, $alpha) = @_;

	unless ($alpha>0) {
	    print STDOUT "[ERROR 'make_weights_smu'] ILLEGAL INPUT: The (minus) power for the weight factor (=$alpha) is NOT POSITIVE: $! !\n";
	    return undef;
	}
	unless (($smu_min>0) and ($smu_min < $smu_max)) {
	    print STDOUT "[ERROR 'make_weights_smu'] ILLEGAL INPUT: The Minimum value (=$smu_min) is NOT WITHIN the open interval, (0, $smu_max): $! !\n";
	    return undef;
	}

	my $ct_smus = @{$smus};
	if ($ct_smus>0) {
	    unless ($smu_min <= $smus->[0]) {
		print STDOUT "[ERROR 'make_weights_smu'] ILLEGAL INPUT: The 0th element of ", '@{$smus} (=', $smus->[0], ") is LESS THAN the specified minimum value (=$smu_min): $! !\n";
		return undef;
	    }
	    unless ($smus->[$ct_smus-1] <= $smu_max) {
		print STDOUT "[ERROR 'make_weights_smu'] ILLEGAL INPUT: The last element of ", '@{$smus} (=', $smus->[$ct_smus-1], ") is GREATER THAN the specified maximum value (=$smu_max): $! !\n";
		return undef;
	    }
	}

	my @boundaries = ($smu_min);
	for (my $k=1; $k<$ct_smus; $k++) { push @boundaries, sqrt ($smus->[$k-1] * $smus->[$k]); }
	push @boundaries, $smu_max;
	my $norm_fac = exp (- $alpha * log ($smu_min)) - exp (- $alpha * log ($smu_max)) ;

	my @weights = ();

	for (my $k=0; $k<$ct_smus; $k++) {
	    my $wgt = exp (-$alpha * log ($boundaries[$k])) - exp (-$alpha * log ($boundaries[$k+1]));
	    $wgt /= $norm_fac;
	    push @weights, $wgt;
	}

	return \@weights;

}	# End of the subroutine 'make_weights_smu' #

